__copyright__ = """

    Copyright 2020 Samapriya Roy

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.

"""
__license__ = "Apache 2.0"
import requests
import csv
from bs4 import BeautifulSoup


def jsext(url, outfile):
    source = requests.get(url)
    html_content = source.text
    soup = BeautifulSoup(html_content, "html.parser")
    try:
        for articles in soup.find_all("script"):
            if not articles.string == None and articles.string.strip().startswith(
                "init"
            ):
                url = articles.string.strip().split('"')[3]
                if url.startswith("https"):
                    iscript = requests.get(url).json()
                    pt = iscript["path"]
                    if outfile == None:
                        print("\n")
                        print(
                            iscript["dependencies"][pt]
                            .encode("utf-8")
                            .decode("utf-8")
                            .strip()
                        )
                    else:
                        file = open(outfile, "w", encoding="utf-8")
                        file.write(str(iscript["dependencies"][pt]).strip())
                        file.close()
                        clean_lines = []
                        with open(outfile, "r", encoding="utf-8") as f:
                            lines = f.readlines()
                            clean_lines = [l.strip("\n") for l in lines if l.strip()]
                        with open(outfile, "w", encoding="utf-8") as f:
                            f.writelines("\n".join(clean_lines))
    except Exception as e:
        print(e)


# jsext(url='https://bullocke.users.earthengine.app/view/amazon',outfile=None)
# jsext(url='https://bullocke.users.earthengine.app/view/amazon')



from __future__ import print_function

__copyright__ = """

    Copyright 2020 Samapriya Roy

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.

"""
__license__ = "Apache 2.0"
import ee
import csv
import random
import subprocess
from logzero import logger

# Empty Lists
folder_paths = []
image_list = []
collection_list = []
table_list = []

suffixes = ["B", "KB", "MB", "GB", "TB", "PB"]


def humansize(nbytes):
    i = 0
    while nbytes >= 1024 and i < len(suffixes) - 1:
        nbytes /= 1024.0
        i += 1
    f = ("%.2f" % nbytes).rstrip("0").rstrip(".")
    return "%s %s" % (f, suffixes[i])


def recprocess(gee_type, location):
    try:
        if gee_type == "collection":
            own = ee.data.getAssetAcl(location)
            o = ",".join(own["owners"])
            r = ",".join(own["readers"])
            w = ",".join(own["writers"])
            return [o, r, w]
        elif gee_type == "image":
            own = ee.data.getAssetAcl(location)
            o = ",".join(own["owners"])
            r = ",".join(own["readers"])
            w = ",".join(own["writers"])
            return [o, r, w]
        elif gee_type == "table":
            own = ee.data.getAssetAcl(location)
            o = ",".join(own["owners"])
            r = ",".join(own["readers"])
            w = ",".join(own["writers"])
            return [o, r, w]
        elif gee_type == "folder":
            own = ee.data.getAssetAcl(location)
            o = ",".join(own["owners"])
            r = ",".join(own["readers"])
            w = ",".join(own["writers"])
            # print(o,r,w)
            return [o, r, w]
    except Exception as e:
        print(e)


# Recursive folder paths
def recursive(path):
    if ee.data.getInfo(path)["type"].lower() == "folder":
        children = ee.data.getList({"id": path})
    folder_paths.append(path)
    val = [child["type"].lower() == "folder" for child in children]
    while len(val) > 0 and True in val:
        for child in children:
            if child["type"].lower() == "folder":
                folder_paths.append(child["id"])
                children = ee.data.getList({"id": child["id"]})
        val = [child["type"].lower() == "folder" for child in children]
    return folder_paths


def assetsize(asset):
    ee.Initialize()
    header = ee.data.getInfo(asset)["type"]
    if header == "IMAGE_COLLECTION":
        collc = ee.ImageCollection(asset)
        size = collc.aggregate_array("system:asset_size")
        return [str(humansize(sum(size.getInfo()))), collc.size().getInfo()]
    elif header == "IMAGE":
        collc = ee.Image(asset)
        return [str(humansize(collc.get("system:asset_size").getInfo())), 1]
    elif header == "TABLE":
        collc = ee.FeatureCollection(asset)
        return [str(humansize(collc.get("system:asset_size").getInfo())), 1]
    elif header == "FOLDER":
        num = subprocess.check_output(
            "earthengine --no-use_cloud_api ls " + asset, shell=True
        ).decode("ascii")
        return ["NA", len(num.split("\n")) - 1]


# folder parse
def fparse(path):
    ee.Initialize()
    if ee.data.getInfo(path)["type"].lower() == "folder":
        gee_folder_path = recursive(path)
        for folders in gee_folder_path:
            children = ee.data.getList({"id": folders})
            for child in children:
                if child["type"].lower() == "imagecollection":
                    collection_list.append(child["id"])
                elif child["type"].lower() == "image":
                    image_list.append(child["id"])
                elif child["type"].lower() == "table":
                    table_list.append(child["id"])
    elif ee.data.getInfo(path)["type"].lower() == "image":
        image_list.append(path)
    elif ee.data.getInfo(path)["type"].lower() == "image_collection":
        collection_list.append(path)
    elif ee.data.getInfo(path)["type"].lower() == "table":
        table_list.append(path)
    else:
        print(ee.data.getInfo(path)["type"].lower())
    return [collection_list, table_list, image_list, folder_paths]


##request type of asset, asset path and user to give permission
def ee_report(output):
    choicelist = [
        "Go grab some tea.....",
        "Go Stretch.....",
        "Go take a walk.....",
        "Go grab some coffee.....",
    ]  # adding something fun
    logger.debug("This might take sometime. {}".format(random.choice(choicelist)))
    ee.Initialize()
    with open(output, "w") as csvfile:
        writer = csv.DictWriter(
            csvfile,
            fieldnames=[
                "type",
                "path",
                "No of Assets",
                "size",
                "owner",
                "readers",
                "writers",
            ],
            delimiter=",",
            lineterminator="\n",
        )
        writer.writeheader()
    collection_path = ee.data.getAssetRoots()

    for roots in collection_path:
        logger.debug(
            "Processing your root folder: {}".format(
                roots["id"].replace("projects/earthengine-legacy/assets/", "")
            )
        )
        collection_list, table_list, image_list, folder_paths = fparse(roots["id"])
        logger.debug(
            "Processing a total of: {} folders {} collections {} images {} tables".format(
                len(folder_paths),
                len(collection_list),
                len(image_list),
                len(table_list),
            )
            + "\n"
        )
        for folder in folder_paths:
            if not folder == roots["id"]:
                gee_id = folder
                gee_type = "folder"
                logger.info("Processing Folder {}".format(gee_id))
                total_size, total_count = assetsize(folder)
                o, r, w = recprocess(gee_type, gee_id)
                # print(gee_id,gee_type,total_size,total_count,o,r,w)
                try:
                    with open(output, "a") as csvfile:
                        writer = csv.writer(csvfile, delimiter=",", lineterminator="\n")
                        writer.writerow(
                            [gee_type, gee_id, total_count, total_size, o, r, w]
                        )
                    csvfile.close()
                except Exception as e:
                    print(e)
        for collection in collection_list:
            gee_id = collection
            gee_type = "collection"
            logger.info("Processing Collection {}".format(gee_id))
            total_size, total_count = assetsize(collection)
            o, r, w = recprocess(gee_type, gee_id)
            # print(gee_id,gee_type,total_size,total_count,o,r,w)
            with open(output, "a") as csvfile:
                writer = csv.writer(csvfile, delimiter=",", lineterminator="\n")
                writer.writerow([gee_type, gee_id, total_count, total_size, o, r, w])
            csvfile.close()
        for table in table_list:
            gee_id = table
            gee_type = "table"
            logger.info("Processing table {}".format(gee_id))
            total_size, total_count = assetsize(table)
            o, r, w = recprocess(gee_type, gee_id)
            # print(gee_id,gee_type,total_size,total_count,o,r,w)
            with open(output, "a") as csvfile:
                writer = csv.writer(csvfile, delimiter=",", lineterminator="\n")
                writer.writerow([gee_type, gee_id, total_count, total_size, o, r, w])
            csvfile.close()
        for image in image_list:
            gee_id = image
            gee_type = "image"
            logger.info("Processing image {}".format(gee_id))
            total_size, total_count = assetsize(image)
            o, r, w = recprocess(gee_type, gee_id)
            # print(gee_id,gee_type,total_size,total_count,o,r,w)
            with open(output, "a") as csvfile:
                writer = csv.writer(csvfile, delimiter=",", lineterminator="\n")
                writer.writerow([gee_type, gee_id, total_count, total_size, o, r, w])
            csvfile.close()



__copyright__ = """

    Copyright 2020 Samapriya Roy

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.

"""
__license__ = "Apache 2.0"
import ee
import json
import itertools

# Empty Lists
folder_paths = []
image_list = []
collection_list = []
table_list = []

# Recursive folder paths
def recursive(path):
    if ee.data.getInfo(path)["type"].lower() == "folder":
        children = ee.data.getList({"id": path})
    folder_paths.append(path)
    val = [child["type"].lower() == "folder" for child in children]
    while len(val) > 0 and True in val:
        for child in children:
            if child["type"].lower() == "folder":
                folder_paths.append(child["id"])
                children = ee.data.getList({"id": child["id"]})
        val = [child["type"].lower() == "folder" for child in children]
    print("Total folders: {}".format(len(folder_paths)))
    return folder_paths


# folder parse
def fparse(path):
    ee.Initialize()
    if ee.data.getInfo(path)["type"].lower() == "folder":
        gee_folder_path = recursive(path)
        for folders in gee_folder_path:
            children = ee.data.getList({"id": folders})
            for child in children:
                if child["type"].lower() == "imagecollection":
                    collection_list.append(child["id"])
                elif child["type"].lower() == "image":
                    image_list.append(child["id"])
                elif child["type"].lower() == "table":
                    table_list.append(child["id"])
    elif ee.data.getInfo(path)["type"].lower() == "image":
        image_list.append(path)
    elif ee.data.getInfo(path)["type"].lower() == "image_collection":
        collection_list.append(path)
    elif ee.data.getInfo(path)["type"].lower() == "table":
        table_list.append(path)
    else:
        print(ee.data.getInfo(path)["type"].lower())
    return [collection_list, table_list, image_list, folder_paths]


##request type of asset, asset path and user to give permission
def access(collection_path, user, role):
    ee.Initialize()
    asset_list = fparse(collection_path)
    asset_names = list(itertools.chain(*asset_list))
    print("Changing permission for total of " + str(len(asset_names)) + " items.....")
    for count, init in enumerate(asset_names):
        print("Working on ===> {}".format(init))
        acl = ee.data.getAssetAcl(init)
        if role == "reader":
            if not user in acl["readers"]:
                baselist = acl["readers"]
                baselist.append(user)
                acl["readers"] = baselist
                acl["owners"] = []
                try:
                    ee.data.setAssetAcl(init, json.dumps(acl))
                except Exception as e:
                    print(e)
            else:
                print("user already has read access to this asset:SKIPPING")
        if role == "writer":
            if not user in acl["writers"]:
                baselist = acl["writers"]
                baselist.append(user)
                acl["readers"] = baselist
                acl["owners"] = []
                try:
                    ee.data.setAssetAcl(init, json.dumps(acl))
                except Exception as e:
                    print(e)
            else:
                print("user already has write access to this asset:SKIPPING")
        if role == "delete":
            if not user in acl["readers"]:
                print("user does not have permission:SKIPPING")
            else:
                baselist = acl["readers"]
                baselist.remove(user)
                acl["readers"] = baselist
                acl["owners"] = []
                try:
                    ee.data.setAssetAcl(init, json.dumps(acl))
                except Exception as e:
                    print(e)



from __future__ import print_function

__copyright__ = """

    Copyright 2020 Samapriya Roy

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.

"""
__license__ = "Apache 2.0"
import ee
import os
import json


# Image copy
def image_move(initial, replace_string, replaced_string, fpath):
    ee.Initialize()
    if replace_string == replaced_string or replace_string == None:
        final = fpath
    else:
        final = initial.replace(replace_string, replaced_string)
    if ee.data.getInfo(final):
        print("Image already move: {}".format(final))
    else:
        print("Moving image to {}".format(final))
        try:
            ee.data.renameAsset(initial, final)
        except Exception as e:
            print(e)


# Table copy
def table_move(initial, replace_string, replaced_string, fpath):
    ee.Initialize()
    if replace_string == replaced_string or replace_string == None:
        final = fpath
    else:
        final = initial.replace(replace_string, replaced_string)
    if ee.data.getInfo(final):
        print("Table already moved: {}".format(final))
    else:
        print("Moving table to {}".format(final))
        try:
            ee.data.renameAsset(initial, final)
        except Exception as e:
            print(e)


# Collection copy
def collection_move(initial, replace_string, replaced_string, fpath):
    ee.Initialize()
    initial_list = ee.data.getList(params={"id": initial})
    assets_names = [os.path.basename(asset["id"]) for asset in initial_list]
    if replace_string == replaced_string or replace_string == None:
        collection_path = fpath
    else:
        collection_path = initial.replace(replace_string, replaced_string)
    if ee.data.getInfo(collection_path):
        print("Collection exists: {}".format(ee.data.getInfo(collection_path)["id"]))
    else:
        print("Collection does not exist: Creating {}".format(collection_path))
        ee.data.createAsset({"type": ee.data.ASSET_TYPE_IMAGE_COLL}, collection_path)
    final_list = ee.data.getList(params={"id": collection_path})
    final_names = [os.path.basename(asset["id"]) for asset in final_list]
    diff = set(assets_names) - set(final_names)
    print("Moving a total of " + str(len(diff)) + " images.....")
    for count, items in enumerate(diff):
        print("Moving " + str(count + 1) + " of " + str(len(diff)), end="\r")
        init = initial + "/" + items
        fin = collection_path + "/" + items
        try:
            ee.data.renameAsset(init, fin)
        except Exception as e:
            print(e)


# Folder create
def fcreate(folder_path, replace_string, replaced_string):
    ee.Initialize()
    folder_path = folder_path.replace(replace_string, replaced_string)
    if ee.data.getInfo(folder_path):
        print("Folder exists: {}".format(ee.data.getInfo(folder_path)["id"]))
    else:
        print("Folder does not exist: Creating {}".format(folder_path))
        ee.data.createAsset({"type": ee.data.ASSET_TYPE_FOLDER}, folder_path)


# Recursive folder paths
folder_paths = []


def recursive(path):
    ee.Initialize()
    if ee.data.getInfo(path)["type"].lower() == "folder":
        children = ee.data.getList({"id": path})
    folder_paths.append(path)
    val = [child["type"].lower() == "folder" for child in children]
    while len(val) > 0 and True in val:
        for child in children:
            if child["type"].lower() == "folder":
                folder_paths.append(child["id"])
                children = ee.data.getList({"id": child["id"]})
        val = [child["type"].lower() == "folder" for child in children]
    print("Total folders: {}".format(len(folder_paths)))
    return folder_paths


# Copy function
def mover(path, fpath):
    ee.Initialize()
    if not ee.data.getInfo(path)==None:
        if ee.data.getInfo(path)["type"].lower() == "folder":
            replace_string=path
            replaced_string=fpath
            gee_folder_path = recursive(path)
            for folders in gee_folder_path:
                fcreate(folders, replace_string, replaced_string)
                children = ee.data.getList({"id": folders})
                for child in children:
                    if child["type"].lower() == "imagecollection":
                        collection_move(child["id"], replace_string, replaced_string, fpath)
                    elif child["type"].lower() == "image":
                        image_move(child["id"], replace_string, replaced_string, fpath)
                    elif child["type"].lower() == "table":
                        table_move(child["id"], replace_string, replaced_string, fpath)
                print("")
        elif ee.data.getInfo(path)["type"].lower() == "image":
            replace_string = None
            replaced_string = "/".join(fpath.split("/")[:-1])
            image_move(path, replace_string, replaced_string, fpath)
        elif ee.data.getInfo(path)["type"].lower() == "image_collection":
            replace_string = None
            replaced_string = "/".join(fpath.split("/")[:-1])
            collection_move(path, replace_string, replaced_string, fpath)
        elif ee.data.getInfo(path)["type"].lower() == "table":
            replace_string = None
            replaced_string = "/".join(fpath.split("/")[:-1])
            table_move(path, replace_string, replaced_string, fpath)
    else:
        print('Initial path {} not found'.format(path))
